/* smb.cmd 
   description: Starts and stops the Samba smbd and nmbd 
                daemons used to provide SMB network services.
 ChangeLog:
 2008-06   : HB: * Add Samba bin and Samba tools to PATH and BEGINLIBPATH 
             HB: * Integrate routines from other Samba tools.
             HB: * Removed /F flag (obsolete)
             HB: * Replaced "smbcontrol ping" check by pstat analysis,
                   as "smbcontrol ping" makes smbd unstable.
             HB: * Merged code from RW and HB
 2008-02-06: RW: * Revised use of '/F' to bypass "smbcontrol ping"
                   and force 'xxxxRunning flags on or off
 2008-01-31: RW: * Major revisions to support different types of Samba
                   setups (self-contained, Unix-like, OS/2-style, etc)
             RW: * Added call to smbsetenv.cmd to configure paths and
                   environment variables (refer to that file for details)
             RW  * Added 'set' & 'reset' environment commands
             RW  * Replaced flag file with more reliable "smbcontrol ping"
             RW  * Expanded 'Usage' screen to be more informative
             RW  * Reorganized code to improve modularity & readability
 2007-12-04: HB: * Made debuglevel configurable.
 2007-11-17: HB: * Now works for builds with and without winbindd.
 2007-10-31: HB: * Start adding winbindd.exe support
 2007-09-27: HB: * make sure neither UNIXROOT nor ETC have a trailing 
                   backslash
 2007-09-03: HB: * smb.conf detection was wrong (UNIXROOT/etc vs ETC)
                 * Add RELOAD as new parameter
 2007-08-22: HB: * Add START /F switch to ignore flagfile
                 * Change into Samba directory before doing something
 2007-08-05: HB: * Don't START Samba in case it is already running
 2007-06-14: HB: * Redirect stderr output to NUL and remove smbstatus nmbd
                   (same output as smbstatus nmbd) in order to make status 
                   useful
                 * Abort if UNIXROOT is missing.
                 * Support logfiles environment variable
                 * Provide errorlevels in case of failure
                 * Updated help text
                 * Added TEST option 
                 * Detect if Samba is not running
 2007-05   : HB: * Initial port from *nix shell script
 */

call _LoadOtherFuncs

/* find out various directories */
parse source . . smbcmd
samba.=''
samba.!tools = strip(filespec('D',smbcmd)||filespec('P',smbcmd),,'\')
smbdexe = stream(samba.!tools'\smbd.exe','c','query exists')
if smbdexe <> "" then samba.!bin = strip(filespec('D',smbdexe)||filespec('P',smbdexe),,'\')
smbdexe = stream(samba.!tools'\..\samba\smbd.exe','c','query exists')
if smbdexe <> "" then samba.!bin = strip(filespec('D',smbdexe)||filespec('P',smbdexe),,'\')
smbdexe = stream(samba.!tools'\..\smbd.exe','c','query exists')
if smbdexe <> "" then samba.!bin = strip(filespec('D',smbdexe)||filespec('P',smbdexe),,'\')
else do
    smbdexe = SysSearchPath("PATH","SMBD.EXE")
    if smbdexe <> "" then samba.!bin = strip(filespec('D',smbdexe)||filespec('P',smbdexe),,'\')
end
if samba.!bin = "" then do
    say "Cannot find smbd.exe - aborting..."
    exit -1
end


OldDir = directory()
call directory samba.!bin
TempDir = strip(value('TMP',,'OS2ENVIRONMENT'),,'\')

UnixRoot = strip(value('UNIXROOT',,'OS2ENVIRONMENT'),,'\')
if UnixRoot = "" then do
    say 'UNIXROOT environment variable is missing. Aborting...'
    ok = directory(OldDir)
    exit -1
end

ETC = strip(value('ETC',,'OS2ENVIRONMENT'),,'\')
if ETC = "" then do
    say 'ETC environment variable is missing. Aborting...'
    ok = directory(OldDir)
    exit -1
end

/* Check that smb.conf exists. */
ok = stream(ETC'\samba\smb.conf','c','query exists')
if ok = "" then do
    say ETC'\samba\smb.conf is missing. Aborting...'
    ok = directory(OldDir)
    exit -2
end
else samba.!smbconf = ok
call _SambaGetProperties
call _SambaExtendSearchPath


/* Check if winbindd daemon exists */
HaveWinbindd = (stream(samba.!bin'\winbindd.exe', 'c', 'query exists') <> "")

/* get the commandline arguments */
arg UArg1 .

select 
    when UArg1 = 'START' then do
        call _SambaStartDaemons
    end
    when UArg1 = 'STOP' then do
        call _SambaStopDaemons
    end
    when UArg1 = 'STATUS' then do
        if \_SambaRunning() then do
            say 'Samba services are not running. Aborting...'
            ok = directory(OldDir)
            exit -3
        end
        'smbstatus smbd 2>NUL'
    end
    when UArg1 = 'RESTART' then do
        say 'Restarting SMB services: '
        call _SambaStopDaemons
        call _SambaStartDaemons
    end
    when UArg1 = 'RELOAD' then do
        say 'Reloading smb.conf.'
        'smbcontrol smbd reload-config 1>NUL'
        say 'Done.'
    end
    when UArg1 = 'TEST' then do
        say 'Testing 'ETC'\Samba\smb.conf'
        'testparm'
        say 'Done.'
    end
    
    otherwise do /* display usage info */
        say 'Samba Server for eCS (OS/2) init script'
        say
        say '  Starts, stops, and controls the Samba services used to'
        say '  provide SMB/CIFS (LAN Server style) network services.'
        say 
        say '  Usage: smb {START | STOP | RESTART | RELOAD | STATUS | TEST }'
        say '   - START    start Samba services'
        say '   - STOP     stop Samba services'
        say '   - RESTART  stop Samba then start it again'
        say '   - RELOAD   put changes to smb.conf into effect'
        say '   - STATUS   show connections & shares in use'
        say '   - TEST     validate smb.conf'
        say
        call charout , '  Samba services are currently '
        if \_SambaRunning() then say 'not running!'; else say 'running!'
        say
        say '  Samba binaries:   'samba.!bin
        say '  Samba tools:      'samba.!tools
        say '  Samba config:     'samba.!smbconf
        say '  Samba logfiles:   'samba.!log_path
        say '  Samba lock files: 'samba.!lock_dir
        say '  Samba debuglevel: 'samba.!debuglevel
        
        ok = directory(OldDir)
        exit 1
    end
end
ok = directory(OldDir)

exit 0

_LoadOtherFuncs:
    '@echo off'

    call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
    call SysLoadFuncs

    call rxfuncadd 'IniLoadFuncs', 'REXXINI', 'IniLoadFuncs'
    call IniLoadFuncs
return

_SambaRunning: procedure expose running. pids. TempDir HaveWinbindd
    /* Checks whether any Samba daemon is currently running
       
       Parameters: none or "ALL" 
       
       Returns: 1 = at least one Samba daemon is running
                1 = Parameter "ALL" all Samba daemons are running
                0 = no Samba daemon is running
                0 = Parameter "ALL" not all Samba daemons are running
                
       Stems: running. stores which daemon is running
              pids.    stores pids of running daemons
              
     */
    restype = arg(1)
    running. = 0
    pids. = ""
        
    pstatout = TempDir||'\pstat.out'
    '@pstat.exe > 'pstatout
    do until lines(pstatout) = 0
        pstatline = linein(pstatout)
        if pos('emapho', pstatline) > 0 then leave /* nothing interesting from here on */
        if pos('SMBD.EXE',pstatline) > 30 then do
            running.!smbd = 1
            pids.!smbd = pids.!smbd||x2d(word(pstatline,1))||', '
        end
        if pos('NMBD.EXE',pstatline) > 30 then do
            running.!nmbd = 1
            pids.!nmbd = pids.!nmbd||x2d(word(pstatline,1))||', '
        end
        if pos('WINBINDD.EXE',pstatline) > 30 then do
            running.!winb = 1
            pids.!winb = pids.!winb||x2d(word(pstatline,1))||', '
        end
    end
    ok = stream(pstatout,'c','close')
    ok = SysFileDelete(PstatOut)
    if restype = "ALL" then runres = (running.!smbd & running.!nmbd & (running.!winb | \HaveWinbindd ))

    runres = (running.!smbd | running.!nmbd |running.!winb)
return runres

_SambaGetProperties: procedure expose samba.
	/* Read some values from smb.conf into samba. stem */
	
    samba.!log_file   = translate(IniGet("log file", "global", samba.!smbconf),'\','/')
    samba.!log_path   = strip(filespec("D",samba.!log_file)||filespec("P",samba.!log_file),,'\')
    samba.!lock_dir   = translate(IniGet("lock directory", "global", samba.!smbconf),'\','/')
    samba.!debuglevel = IniGet("log level", "global", samba.!smbconf)
    
    ok = value("SMB_LOGS",samba.!log_path,"OS2ENVIRONMENT")
    ok = value("SMB_LOCK",samba.!lock_dir,"OS2ENVIRONMENT")
    
    address cmd samba.!bin'\smbd.exe -V >smb.ver'
    samba.!version = linein('smb.ver')
    ok = stream('smb.ver','c','close')
    ok = SysFileDelete('smb.ver')
return

_SambaStopDaemons:
    say 'Shutting down SMB services'
    Red='1B'x||'[31;m'
    Green='1B'x||'[32;m'
    Normal='1B'x||'[0m'
    
    ok = _SambaRunning()

    if running.!smbd = 1 then do
        'smbcontrol smbd shutdown >' samba.!log_path'\log.smbd.shtdn 2>&1'
        ok = SysFileDelete(ETC'\samba\pid\smbd.pid')
    end

    if running.!nmbd = 1 then do
        'smbcontrol nmbd shutdown >' samba.!log_path'\log.nmbd.shtdn 2>&1'
        ok = SysFileDelete(ETC'\samba\pid\nmbd.pid')
    end

    if running.!winb = 1 then do
        'smbcontrol winbindd shutdown >' samba.!log_path'\log.winbindd.shtdn 2>&1'
        ok = SysFileDelete(ETC'\samba\pid\winbindd.pid')
    end
    ok = SysSleep(0.1)
    if \_SambaRunning() then say Green||"Samba "samba.!version" services stopped successfully."||Normal
    else do
        if running.!smbd = 1 then say Red||"Warning! An instance of smbd.exe is still running."||Normal
        if running.!nmbd = 1 then say Red||"Warning! An instance of nmbd.exe is still running."||Normal
        if running.!winb = 1 then say Red||"Warning! An instance of winbindd.exe is still running."||Normal
    end

    /* Plugin script to be executed after Samba is stopped */
    smbexit = stream(samba.!tools'\smbexit.cmd', 'c', 'query exists')
    if smbexit <> '' then do
        say 'Finishing shutdown using smbexit.cmd'
        interpret '"call 'smbexit'";'
        drop smbexit
    end
    say 'Done.'
return

_SambaStartDaemons:
    Red='1B'x||'[31;m'
    Green='1B'x||'[32;m'
    Normal='1B'x||'[0m'

    if _SambaRunning() then do
        say "Samba services are already running."
        return
    end
    /* Plugin script to be executed before Samba is started */
    b4smb = stream(samba.!tools'\b4smb.cmd', 'c', 'query exists')
    if b4smb <> '' then do
        say 'Configuring SMB services using b4smb.cmd'
        interpret '"call 'b4smb'";'
        drop b4smb
    end

    say 'Starting SMB services'

    if running.!smbd = 0 then do
        ok = SysFileDelete(ETC'\samba\pid\smbd.pid')
        ok = charout(, '  SMBD:     ')
        'detach smbd'
    end
    if running.!nmbd = 0 then do
        ok = SysFileDelete(etcpath'\samba\pid\nmbd.pid')
        ok = charout(, '  NMBD:     ')
        'detach nmbd -l 'samba.!log_path' -d 'samba.!debugLevel
    end
    if haveWinbindd & running.!winb = 0 then do
        ok = charout(, '  WINBINDD: ')
        'detach winbindd -l 'samba.!log_path' -d 'samba.!debugLevel
    end
    ok = SysSleep(0.2)
    if _SambaRunning("ALL") then say Green||"Samba "samba.!version" services started successfully."||Normal
    else do
        if running.!smbd = 0 then say Red||"Warning! smbd.exe apparently stopped."||Normal
        if running.!nmbd = 0 then say Red||"Warning! nmbd.exe apparently stopped."||Normal
     /* if running.!winb = 0 then say Red||"Warning! winbindd.exe apparently stopped."||Normal */
    end

    say 'Done.'
return

_SambaExtendSearchPath:
    /* Add binary and tools path to the PATH variable */
    old_path = value('PATH',, 'OS2ENVIRONMENT')
    if pos(translate(samba.!bin';'),translate(old_path)) = 0 then do
        if samba.!bin = samba.!tools then new_path = samba.!bin';'
        else new_path = samba.!bin';'samba.!tools';'
        ok = value('PATH', new_path || old_path, 'OS2ENVIRONMENT')
        drop new_path
    end
    drop old_path
    
    /* Add binary and tools path to the BEGINLIBPATH variable */
    old_beginlibpath = SysQueryEXtLibPath("B")
    if pos(translate(samba.!bin';'),translate(old_beginlibpath)) = 0 then do
        if samba.!bin = samba.!tools then new_beginlibpath = samba.!bin';'old_beginlibpath
        else new_beginlibpath = samba.!bin';'samba.!tools';'old_beginlibpath
        ok = SysSetExtLibPath( new_beginlibpath, "B")
        drop new_beginlibpath
    end
    drop old_beginlibpath
return

